@@ -12,7 +12,7 @@ class pqMyApplicationStarter : public QObject
   typedef QObject Superclass;
 
 public:
-  pqMyApplicationStarter(QObject* p = 0);
+  pqMyApplicationStarter(QObject* parent = nullptr);
   ~pqMyApplicationStarter();
 
   // Callback for shutdown.
@@ -21,6 +21,9 @@ public:
   // Callback for startup.
   void onStartup();
 
+public Q_SLOTS:
+  void playAnimation();
+
 private:
   Q_DISABLE_COPY(pqMyApplicationStarter)
 };
