@@ -14,7 +14,7 @@
 #include "vtkObjectFactory.h"
 #include "vtkPVDataInformation.h"
 #include "vtkPVSessionBase.h"
-#include "vtkPVVersion.h"
+#include "vtkPVVersionQuick.h"
 #include "vtkPVXMLElement.h"
 #include "vtkPVXMLParser.h"
 #include "vtkProcessModule.h"
@@ -38,7 +38,7 @@
 #include <vtksys/SystemInformation.hxx>
 #include <vtksys/SystemTools.hxx>
 
-//#define vtkLiveInsituLinkDebugMacro(x) cerr x << endl;
+// #define vtkLiveInsituLinkDebugMacro(x) cerr x << endl;
 #define vtkLiveInsituLinkDebugMacro(x)
 
 namespace
@@ -58,7 +58,7 @@ void TriggerRMI(vtkMultiProcessController* controller, int tag, double time, vtk
   unsigned char* data = nullptr;
   unsigned int dataSize;
   stream.GetRawData(data, dataSize);
-  controller->TriggerRMI(1, const_cast<unsigned char*>(data), dataSize, tag);
+  controller->TriggerRMI(1, data, dataSize, tag);
   delete[] data;
 }
 
@@ -70,7 +70,7 @@ void TriggerRMIOnAllChildren(
   unsigned char* data = nullptr;
   unsigned int dataSize;
   stream.GetRawData(data, dataSize);
-  controller->TriggerRMIOnAllChildren(const_cast<unsigned char*>(data), dataSize, tag);
+  controller->TriggerRMIOnAllChildren(data, dataSize, tag);
   delete[] data;
 }
 
@@ -451,6 +451,7 @@ bool vtkLiveInsituLink::InitializeInsitu()
     vtkNetworkAccessManager* nam = pm->GetNetworkAccessManager();
 
     std::ostringstream url;
+    url.imbue(std::locale::classic());
     url << "tcp://" << this->Hostname << ":" << this->InsituPort << "?"
         << "timeout=0&handshake=paraview.insitu." << PARAVIEW_VERSION;
     this->SetURL(url.str().c_str());
